# SPDX-FileCopyrightText: 2025 German Aerospace Center <amiris@dlr.de>
#
# SPDX-License-Identifier: Apache-2.0

"""Hosts exceptions thrown by AMIRIS-Py."""


class AMIRISError(Exception):
    """An error occurred during the execution of AMIRIS-Py."""
